/*
 * Copyright (c) 2006 David Rudie
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <pspkernel.h>
#include <pspdebug.h>
#include <pspsdk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <pspwlan.h>
#include <pspnet.h>
#include <pspnet_inet.h>
#include <pspnet_apctl.h>
#include <pspnet_resolver.h>
#include <psppower.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/select.h>
#include <errno.h>
#include <pspdisplay.h>
#include <pspctrl.h>

#define printf pspDebugScreenPrintf

#define MODULE_NAME "WifiController"

// #define MAXPATHLEN 512

PSP_MODULE_INFO(MODULE_NAME, 0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0);

int done = 0;

int exit_callback(int arg1, int arg2, void *common) {
  done = 1;
  sceKernelExitGame();
  return 0;
}

int CallbackThread(SceSize args, void *argp) {
  int cbid;

  cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
  sceKernelRegisterExitCallback(cbid);
  sceKernelSleepThreadCB();

  return 0;
}

int SetupCallbacks(void) {
  int thid = 0;

  thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, PSP_THREAD_ATTR_USER, 0);
  if (thid >= 0) {
    sceKernelStartThread(thid, 0, 0);
  }

  return thid;
}

int ConnectToClient(void) {
  int sock;
  struct sockaddr_in addr;
  int err;
  char buf[MAXPATHLEN];
  char file_path[128];
  char Buffer[512];
  char *Scan;
  FILE *CfgFile;
  char host[32];
  unsigned short port = 0;
  char temp[10];

  pspDebugScreenClear();

  strcpy(file_path, getcwd(buf, MAXPATHLEN));
  strcat(file_path, "/wifi.cfg");

  printf("Reading configuration file: %s \n", file_path);

  CfgFile = fopen(file_path, "r");

  if (CfgFile != (FILE*)0) {
    while (fgets(Buffer, 512, CfgFile) != (char *)0) {
      Scan = strchr(Buffer, '\n');
      if (Scan) *Scan = '\0';
      Scan = strchr(Buffer, '\r');
      if (Scan) *Scan = '\0';

      if (Buffer[0] == '#') continue;

      if (!strncasecmp(Buffer, "address=", 8)) {
        printf("Read: address=%s\n", Buffer + 8);
        strcpy(host, Buffer + 8);
        printf("host = %s\n", host);
      }
      if (!strncasecmp(Buffer, "port=", 5)) {
        printf("Read: port=%s\n", Buffer + 5);
        strcpy(temp, Buffer + 5);
        port = atoi(temp);
        printf("port=%d\n", port);
      }
    }
    fclose(CfgFile);
  }

  sceKernelDelayThread(2000*1000);

  memset(&addr, 0, sizeof(struct sockaddr_in));
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  sceNetInetInetAton(host, &addr.sin_addr);

  sock = sceNetInetSocket(AF_INET, SOCK_DGRAM, 0);
  if (sock < 0) {
    return -1;
  }

  err = sceNetInetConnect(sock, &addr, sizeof(addr));
  if (err) {
    return -1;
  }

  return sock;
}

void read_pad(void) {
  int sock;
  int batteryLifeTime = 0;

  SceCtrlData pad;
  unsigned int packetNum = 1;
  unsigned char packet[11];
  memset(packet, 0, sizeof(packet));

  sock = ConnectToClient();
  if (sock < 0) {
    printf("Error opening socket\n");
    return;
  }

  sceCtrlSetSamplingCycle(0);
  sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);

  pspDebugScreenClear();

  while (!done) {
    if (sceWlanGetSwitchState() != 1) {
      connect_to_apctl(1);
      read_pad();
    }
    pspDebugScreenSetXY(0, 2);
    sceCtrlReadBufferPositive(&pad, 1);
    printf("        Analog X   %d      \n", pad.Lx);
    printf("        Analog Y   %d      \n\n", pad.Ly);

/*
      Analog X   111
      Analog Y   111

  k                  q

                    
                  
                    V

      SELECT   START     
*/

    if (pad.Buttons & PSP_CTRL_LTRIGGER) { printf("    "); } else { printf("    k"); }
    if (pad.Buttons & PSP_CTRL_RTRIGGER) { printf("                    \n\n"); } else { printf("                  q  \n\n"); }

    if (pad.Buttons & PSP_CTRL_UP)       { printf("    "); } else { printf("    "); }
    if (pad.Buttons & PSP_CTRL_TRIANGLE) { printf("                    \n"); } else { printf("                    \n"); }
    if (pad.Buttons & PSP_CTRL_LEFT)     { printf("  "); } else { printf("  "); }
    if (pad.Buttons & PSP_CTRL_RIGHT)    { printf("  "); } else { printf("  "); }
    if (pad.Buttons & PSP_CTRL_SQUARE)   { printf("              "); } else { printf("              "); }
    if (pad.Buttons & PSP_CTRL_CIRCLE)   { printf("  \n"); } else { printf("  \n"); }
    if (pad.Buttons & PSP_CTRL_DOWN)     { printf("    "); } else { printf("    "); }
    if (pad.Buttons & PSP_CTRL_CROSS)    { printf("                    \n\n"); } else { printf("                  ~  \n\n"); }
 
    if (pad.Buttons & PSP_CTRL_SELECT)   { printf("      SELECT"); } else { printf("        SELECT"); }
    if (pad.Buttons & PSP_CTRL_START)    { printf("   START   \n"); } else { printf("   START     \n"); }
    printf("\n");
    if (scePowerIsBatteryExist()) {
      batteryLifeTime = scePowerGetBatteryLifeTime();
      printf("Battery Life: %d%% (%02dh%02dm) - Battery Temperature: %d C\n", scePowerGetBatteryLifePercent(), batteryLifeTime / 60, batteryLifeTime - (batteryLifeTime / 60 * 60), scePowerGetBatteryTemp());
    }
    //printf("Buttons = %08x\n", pad.Buttons);
    /* The code below was written by Mikko Monone. */
    unsigned char packet[11];
    packet[0] = '!';
    packet[1] = 'K';
    packet[2] = (packetNum) & 0xff;
    packet[3] = (packetNum >> 8) & 0xff;
    packet[4] = (packetNum >> 16) & 0xff;
    packet[5] = (packetNum >> 24) & 0xff;
    packet[6] = pad.Lx;
    packet[7] = pad.Ly;
    packet[8] = (pad.Buttons) & 0xff;
    packet[9] = (pad.Buttons >> 8) & 0xff;
    packet[10] = (pad.Buttons >> 16) & 0xff;
    packetNum++;
    /* The code above was written by Mikko Monone. */
    sceNetInetSend(sock, packet, sizeof(packet), 0);
    sceDisplayWaitVblankStart();
  }
}

int connect_to_apctl(int config) {
  int err;
  int stateLast = -1;

  while (sceWlanGetSwitchState() != 1) {
    printf("Please enable WLAN to continue.\n");
    sceKernelDelayThread(1000*1000);
  }

  err = sceNetApctlConnect(config);
  if (err != 0) {
    printf(MODULE_NAME ": sceNetApctlConnect returns %08X\n", err);
    return 0;
  }

  printf(MODULE_NAME ": Connecting...\n");
  while (1) {
    int state;
    err = sceNetApctlGetState(&state);
    if (err != 0) {
      printf(MODULE_NAME ": sceNetApctlGetState returns $%x\n", err);
      break;
    }
    if (state > stateLast) {
      printf("  connection state %d of 4\n", state);
      stateLast = state;
    }
    if (state == 4)
      break;

    sceKernelDelayThread(50*1000);
  }
  printf(MODULE_NAME ": Connected!\n");

  if (err != 0) {
    return 0;
  }

  return 1;
}

int net_thread(SceSize args, void *argp) {
  int err;

  do {
    if ((err = pspSdkInetInit())) {
      printf(MODULE_NAME ": Error, could not initialise the network %08X\n", err);
      break;
    }

    if (connect_to_apctl(1)) {
      char szMyIPAddr[32];
      if (sceNetApctlGetInfo(8, szMyIPAddr) != 0)
        strcpy(szMyIPAddr, "unknown IP address");

      read_pad();
    }
  }
  while(0);

  return 0;
}

int main(int argc, char **argv) {
  SceUID thid;

  SetupCallbacks();

  pspDebugScreenInit();

  printf("WiFiController v0.2\nCopyright (c)2006 David Rudie\n");
  sceKernelDelayThread(3000*1000);
  pspDebugScreenClear();

  if (pspSdkLoadInetModules() < 0) {
    printf("Error, could not load inet modules\n");
    sceKernelSleepThread();
  }

  thid = sceKernelCreateThread("net_thread", net_thread, 0x18, 0x10000, PSP_THREAD_ATTR_USER, NULL);
  if (thid < 0) {
    printf("Error, could not create thread\n");
    sceKernelSleepThread();
  }

  sceKernelStartThread(thid, 0, NULL);

  sceKernelExitDeleteThread(0);

  return 0;
}
